import { Router, Request, Response } from 'express';
import { authenticateToken, requireDeveloper } from '@/middleware/auth';
import { validate, validateQuery, createProjectSchema, updateProjectSchema, paginationSchema } from '@/middleware/validation';
import { prisma } from '@/utils/database';
import { CreateProjectRequest, UpdateProjectRequest, ApiResponse, PaginationOptions, FilterOptions } from '@/types';
import { VerificationStatus } from '@prisma/client';

const router = Router();

// Create project endpoint (Developer only)
router.post('/', authenticateToken, requireDeveloper, validate(createProjectSchema), async (req: Request, res: Response) => {
  try {
    const projectData: CreateProjectRequest = req.body;
    const developerId = req.user!.id;

    // Create project
    const project = await prisma.carbonProject.create({
      data: {
        ...projectData,
        developerId
      }
    });

    // Create initial carbon offset
    await prisma.carbonOffset.create({
      data: {
        serialNumber: `CO${project.id}-${projectData.totalCreditsAvailable}`,
        vintageYear: new Date(projectData.startDate).getFullYear(),
        quantity: projectData.totalCreditsAvailable,
        projectId: project.id
      }
    });

    return res.status(201).json({
      success: true,
      data: project,
      message: 'Project created successfully'
    } as ApiResponse);

  } catch (error) {
    console.error('Project creation error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during project creation'
    } as ApiResponse);
  }
});

// List all projects endpoint
router.get('/', validateQuery(paginationSchema), async (req: Request, res: Response) => {
  try {
    const { page = 1, pageSize = 10 } = req.query as PaginationOptions;
    const { projectType, verificationStatus } = req.query as FilterOptions;

    // Build where clause
    const where: any = { isActive: true };
    if (projectType) where.projectType = projectType;
    if (verificationStatus) where.verificationStatus = verificationStatus as VerificationStatus;

    // Get total count
    const total = await prisma.carbonProject.count({ where });

    // Get projects with pagination
    const projects = await prisma.carbonProject.findMany({
      where,
      orderBy: { createdAt: 'desc' },
      skip: (page - 1) * pageSize,
      take: pageSize,
      include: {
        developer: {
          select: {
            id: true,
            fullName: true,
            email: true
          }
        }
      }
    });

    return res.status(200).json({
      success: true,
      data: {
        projects,
        total,
        page,
        pageSize
      }
    } as ApiResponse);

  } catch (error) {
    console.error('Projects list error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during projects retrieval'
    } as ApiResponse);
  }
});

// Get my projects endpoint (Developer only)
router.get('/my-projects', authenticateToken, requireDeveloper, validateQuery(paginationSchema), async (req: Request, res: Response) => {
  try {
    const { page = 1, pageSize = 10 } = req.query as PaginationOptions;
    const developerId = req.user!.id;

    // Get total count
    const total = await prisma.carbonProject.count({
      where: { developerId }
    });

    // Get projects with pagination
    const projects = await prisma.carbonProject.findMany({
      where: { developerId },
      orderBy: { createdAt: 'desc' },
      skip: (page - 1) * pageSize,
      take: pageSize
    });

    return res.status(200).json({
      success: true,
      data: {
        projects,
        total,
        page,
        pageSize
      }
    } as ApiResponse);

  } catch (error) {
    console.error('My projects error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during projects retrieval'
    } as ApiResponse);
  }
});

// Get specific project endpoint
router.get('/:projectId', async (req: Request, res: Response) => {
  try {
    const projectId = parseInt(req.params.projectId);

    if (isNaN(projectId)) {
      return res.status(400).json({
        success: false,
        error: 'Invalid project ID'
      } as ApiResponse);
    }

    const project = await prisma.carbonProject.findFirst({
      where: {
        id: projectId,
        isActive: true
      },
      include: {
        developer: {
          select: {
            id: true,
            fullName: true,
            email: true
          }
        },
        offsets: {
          select: {
            id: true,
            serialNumber: true,
            quantity: true,
            status: true
          }
        }
      }
    });

    if (!project) {
      return res.status(404).json({
        success: false,
        error: 'Project not found'
      } as ApiResponse);
    }

    return res.status(200).json({
      success: true,
      data: project
    } as ApiResponse);

  } catch (error) {
    console.error('Project retrieval error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during project retrieval'
    } as ApiResponse);
  }
});

// Update project endpoint (Developer only - own projects)
router.put('/:projectId', authenticateToken, requireDeveloper, validate(updateProjectSchema), async (req: Request, res: Response) => {
  try {
    const projectId = parseInt(req.params.projectId);
    const updateData: UpdateProjectRequest = req.body;
    const developerId = req.user!.id;

    if (isNaN(projectId)) {
      return res.status(400).json({
        success: false,
        error: 'Invalid project ID'
      } as ApiResponse);
    }

    // Check if project exists and belongs to user
    const existingProject = await prisma.carbonProject.findFirst({
      where: {
        id: projectId,
        developerId
      }
    });

    if (!existingProject) {
      return res.status(404).json({
        success: false,
        error: 'Project not found or access denied'
      } as ApiResponse);
    }

    // Update project
    const project = await prisma.carbonProject.update({
      where: { id: projectId },
      data: updateData
    });

    return res.status(200).json({
      success: true,
      data: project,
      message: 'Project updated successfully'
    } as ApiResponse);

  } catch (error) {
    console.error('Project update error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during project update'
    } as ApiResponse);
  }
});

// Delete project endpoint (Developer only - own projects)
router.delete('/:projectId', authenticateToken, requireDeveloper, async (req: Request, res: Response) => {
  try {
    const projectId = parseInt(req.params.projectId);
    const developerId = req.user!.id;

    if (isNaN(projectId)) {
      return res.status(400).json({
        success: false,
        error: 'Invalid project ID'
      } as ApiResponse);
    }

    // Check if project exists and belongs to user
    const existingProject = await prisma.carbonProject.findFirst({
      where: {
        id: projectId,
        developerId
      }
    });

    if (!existingProject) {
      return res.status(404).json({
        success: false,
        error: 'Project not found or access denied'
      } as ApiResponse);
    }

    // Soft delete - mark as inactive
    await prisma.carbonProject.update({
      where: { id: projectId },
      data: { isActive: false }
    });

    return res.status(200).json({
      success: true,
      message: 'Project deleted successfully'
    } as ApiResponse);

  } catch (error) {
    console.error('Project deletion error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during project deletion'
    } as ApiResponse);
  }
});

export default router;