import { Router, Request, Response } from 'express';
import { authenticateToken, requireBuyer } from '@/middleware/auth';
import { validate, validateQuery, purchaseSchema, paginationSchema } from '@/middleware/validation';
import { prisma } from '@/utils/database';
import { PurchaseRequest, ApiResponse, PaginationOptions, FilterOptions } from '@/types';
import { TransactionStatus, OffsetStatus } from '@prisma/client';
import { v4 as uuidv4 } from 'uuid';

const router = Router();

// Purchase carbon offsets endpoint (Buyer only)
router.post('/purchase', authenticateToken, requireBuyer, validate(purchaseSchema), async (req: Request, res: Response) => {
  try {
    const { projectId, quantity }: PurchaseRequest = req.body;
    const buyerId = req.user!.id;

    // Check if user has wallet linked
    const user = await prisma.user.findUnique({
      where: { id: buyerId }
    });

    if (!user?.walletAddress) {
      return res.status(400).json({
        success: false,
        error: 'Wallet must be linked to purchase carbon offsets'
      } as ApiResponse);
    }

    // Get project
    const project = await prisma.carbonProject.findFirst({
      where: {
        id: projectId,
        isActive: true,
        verificationStatus: 'VERIFIED'
      }
    });

    if (!project) {
      return res.status(404).json({
        success: false,
        error: 'Project not found or not verified'
      } as ApiResponse);
    }

    // Check if enough credits are available
    const availableCredits = project.totalCreditsAvailable - project.creditsSold;
    if (quantity > availableCredits) {
      return res.status(400).json({
        success: false,
        error: `Not enough credits available. Available: ${availableCredits}`
      } as ApiResponse);
    }

    // Get available offset
    const offset = await prisma.carbonOffset.findFirst({
      where: {
        projectId,
        status: OffsetStatus.AVAILABLE
      }
    });

    if (!offset) {
      return res.status(400).json({
        success: false,
        error: 'No available carbon offsets for this project'
      } as ApiResponse);
    }

    // Calculate total amount
    const totalAmount = quantity * project.pricePerCredit;

    // Create transaction in a database transaction
    const result = await prisma.$transaction(async (tx) => {
      // Create transaction record
      const transactionHash = `tx_${uuidv4().replace(/-/g, '').substring(0, 16)}`;
      
      const transaction = await tx.transaction.create({
        data: {
          transactionHash,
          quantity,
          pricePerCredit: project.pricePerCredit,
          totalAmount,
          buyerId,
          offsetId: offset.id,
          status: TransactionStatus.PENDING
        }
      });

      // Update project credits sold
      await tx.carbonProject.update({
        where: { id: projectId },
        data: {
          creditsSold: { increment: quantity }
        }
      });

      // Update offset quantity or status
      if (offset.quantity <= quantity) {
        await tx.carbonOffset.update({
          where: { id: offset.id },
          data: { status: OffsetStatus.SOLD }
        });
      } else {
        // Update existing offset and create new offset for sold quantity
        await tx.carbonOffset.update({
          where: { id: offset.id },
          data: { quantity: { decrement: quantity } }
        });

        await tx.carbonOffset.create({
          data: {
            serialNumber: `CO${projectId}-sold-${transaction.id}`,
            vintageYear: offset.vintageYear,
            quantity,
            status: OffsetStatus.SOLD,
            projectId
          }
        });
      }

      // Simulate blockchain transaction confirmation
      // In a real implementation, you would integrate with actual blockchain
      const confirmedTransaction = await tx.transaction.update({
        where: { id: transaction.id },
        data: {
          status: TransactionStatus.CONFIRMED,
          confirmedAt: new Date(),
          blockNumber: Math.floor(Math.random() * 1000000) + 12000000, // Simulated
          gasUsed: 21000 // Simulated
        }
      });

      return confirmedTransaction;
    });

    return res.status(201).json({
      success: true,
      data: result,
      message: 'Carbon offsets purchased successfully'
    } as ApiResponse);

  } catch (error) {
    console.error('Purchase error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during purchase'
    } as ApiResponse);
  }
});

// Get user's transactions endpoint
router.get('/my-transactions', authenticateToken, validateQuery(paginationSchema), async (req: Request, res: Response) => {
  try {
    const { page = 1, pageSize = 10 } = req.query as PaginationOptions;
    const { status } = req.query as FilterOptions;
    const buyerId = req.user!.id;

    // Build where clause
    const where: any = { buyerId };
    if (status) where.status = status as TransactionStatus;

    // Get total count
    const total = await prisma.transaction.count({ where });

    // Get transactions with pagination
    const transactions = await prisma.transaction.findMany({
      where,
      orderBy: { createdAt: 'desc' },
      skip: (page - 1) * pageSize,
      take: pageSize,
      include: {
        offset: {
          include: {
            project: {
              select: {
                id: true,
                title: true,
                projectType: true,
                location: true
              }
            }
          }
        }
      }
    });

    return res.status(200).json({
      success: true,
      data: {
        transactions,
        total,
        page,
        pageSize
      }
    } as ApiResponse);

  } catch (error) {
    console.error('Transactions retrieval error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during transactions retrieval'
    } as ApiResponse);
  }
});

// Get specific transaction endpoint
router.get('/transactions/:transactionId', authenticateToken, async (req: Request, res: Response) => {
  try {
    const transactionId = parseInt(req.params.transactionId);
    const userId = req.user!.id;

    if (isNaN(transactionId)) {
      return res.status(400).json({
        success: false,
        error: 'Invalid transaction ID'
      } as ApiResponse);
    }

    const transaction = await prisma.transaction.findFirst({
      where: {
        id: transactionId,
        buyerId: userId
      },
      include: {
        offset: {
          include: {
            project: {
              select: {
                id: true,
                title: true,
                projectType: true,
                location: true,
                developer: {
                  select: {
                    fullName: true,
                    email: true
                  }
                }
              }
            }
          }
        }
      }
    });

    if (!transaction) {
      return res.status(404).json({
        success: false,
        error: 'Transaction not found'
      } as ApiResponse);
    }

    return res.status(200).json({
      success: true,
      data: transaction
    } as ApiResponse);

  } catch (error) {
    console.error('Transaction retrieval error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during transaction retrieval'
    } as ApiResponse);
  }
});

// Get marketplace statistics endpoint
router.get('/marketplace/stats', async (req: Request, res: Response) => {
  try {
    // Get marketplace statistics
    const [
      totalProjects,
      verifiedProjects,
      totalTransactions,
      totalVolumeResult,
      availableCreditsResult
    ] = await Promise.all([
      prisma.carbonProject.count({
        where: { isActive: true }
      }),
      prisma.carbonProject.count({
        where: {
          isActive: true,
          verificationStatus: 'VERIFIED'
        }
      }),
      prisma.transaction.count({
        where: { status: TransactionStatus.CONFIRMED }
      }),
      prisma.transaction.aggregate({
        where: { status: TransactionStatus.CONFIRMED },
        _sum: { totalAmount: true }
      }),
      prisma.carbonProject.aggregate({
        where: { isActive: true },
        _sum: {
          totalCreditsAvailable: true,
          creditsSold: true
        }
      })
    ]);

    const totalCreditsAvailable = (availableCreditsResult._sum.totalCreditsAvailable || 0) - 
                                 (availableCreditsResult._sum.creditsSold || 0);
    const totalVolumeTraded = totalVolumeResult._sum.totalAmount || 0;

    const stats = {
      totalProjects,
      verifiedProjects,
      totalCreditsAvailable,
      totalTransactions,
      totalVolumeTraded
    };

    return res.status(200).json({
      success: true,
      data: stats
    } as ApiResponse);

  } catch (error) {
    console.error('Marketplace stats error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during marketplace stats retrieval'
    } as ApiResponse);
  }
});

export default router;