import { Router, Request, Response } from 'express';
import { authenticateToken } from '@/middleware/auth';
import { validate, walletLinkSchema } from '@/middleware/validation';
import { walletService } from '@/services/wallet';
import { WalletLinkRequest, ApiResponse } from '@/types';

const router = Router();

// All wallet routes require authentication
router.use(authenticateToken);

// Link wallet endpoint
router.post('/link', validate(walletLinkSchema), async (req: Request, res: Response) => {
  try {
    const { walletAddress }: WalletLinkRequest = req.body;
    const userId = req.user!.id;

    const result = await walletService.linkWallet(userId, walletAddress);

    if (!result.success) {
      return res.status(400).json({
        success: false,
        error: result.message
      } as ApiResponse);
    }

    return res.status(200).json({
      success: true,
      data: result,
      message: result.message
    } as ApiResponse);

  } catch (error) {
    console.error('Wallet link error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during wallet linking'
    } as ApiResponse);
  }
});

// Unlink wallet endpoint
router.delete('/unlink', async (req: Request, res: Response) => {
  try {
    const userId = req.user!.id;

    const result = await walletService.unlinkWallet(userId);

    if (!result.success) {
      return res.status(400).json({
        success: false,
        error: result.message
      } as ApiResponse);
    }

    return res.status(200).json({
      success: true,
      data: result,
      message: result.message
    } as ApiResponse);

  } catch (error) {
    console.error('Wallet unlink error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during wallet unlinking'
    } as ApiResponse);
  }
});

// Get wallet info endpoint
router.get('/info', async (req: Request, res: Response) => {
  try {
    const userId = req.user!.id;

    const result = await walletService.getWalletInfo(userId);

    if (!result.success) {
      return res.status(400).json({
        success: false,
        error: result.message
      } as ApiResponse);
    }

    return res.status(200).json({
      success: true,
      data: result
    } as ApiResponse);

  } catch (error) {
    console.error('Wallet info error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during wallet info retrieval'
    } as ApiResponse);
  }
});

// Generate test wallet endpoint
router.post('/generate-test-wallet', async (req: Request, res: Response) => {
  try {
    const result = walletService.generateTestWallet();

    return res.status(200).json({
      success: true,
      data: result,
      message: result.message
    } as ApiResponse);

  } catch (error) {
    console.error('Test wallet generation error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during test wallet generation'
    } as ApiResponse);
  }
});

export default router;