import express, { Request, Response } from 'express';
import compression from 'compression';
import morgan from 'morgan';
import dotenv from 'dotenv';
import Database from '@/utils/database';
import { setupSecurity } from '@/middleware/security';
import { errorHandler, notFoundHandler } from '@/middleware/error';
import { blockchainService } from '@/services/blockchain';
import { ApiResponse } from '@/types';

// Import routes
import authRoutes from '@/routes/auth';
import walletRoutes from '@/routes/wallet';
import projectRoutes from '@/routes/projects';
import tradingRoutes from '@/routes/trading';

// Load environment variables
dotenv.config();

class Server {
  private app: express.Application;
  private port: number;

  constructor() {
    this.app = express();
    this.port = parseInt(process.env.PORT || '8000');
    
    this.setupMiddleware();
    this.setupRoutes();
    this.setupErrorHandling();
  }

  private setupMiddleware(): void {
    // Security middleware
    setupSecurity(this.app);
    
    // Compression middleware
    this.app.use(compression());
    
    // Logging middleware
    if (process.env.NODE_ENV !== 'test') {
      this.app.use(morgan('combined'));
    }
    
    // Body parsing middleware
    this.app.use(express.json({ limit: '10mb' }));
    this.app.use(express.urlencoded({ extended: true, limit: '10mb' }));
  }

  private setupRoutes(): void {
    // Health check endpoint
    this.app.get('/health', async (req: Request, res: Response) => {
      try {
        // Check database connection
        const dbHealthy = await Database.healthCheck();
        
        // Check blockchain connection (optional)
        const blockchainHealthy = await blockchainService.isNetworkConnected();
        
        // Check environment variables
        const requiredEnvVars = ['JWT_SECRET'];
        const missingEnvVars = requiredEnvVars.filter(env => !process.env[env]);
        const envHealthy = missingEnvVars.length === 0;
        
        const isHealthy = dbHealthy && envHealthy;
        
        const healthData = {
          status: isHealthy ? 'healthy' : 'unhealthy',
          timestamp: new Date().toISOString(),
          version: '1.0.0',
          environment: process.env.NODE_ENV || 'development',
          services: {
            database: dbHealthy ? 'healthy' : 'unhealthy',
            blockchain: blockchainHealthy ? 'connected' : 'disconnected',
            environment: envHealthy ? 'healthy' : `missing: ${missingEnvVars.join(', ')}`
          }
        };
        
        return res.status(isHealthy ? 200 : 503).json({
          success: isHealthy,
          data: healthData
        } as ApiResponse);
        
      } catch (error) {
        console.error('Health check error:', error);
        return res.status(503).json({
          success: false,
          error: 'Health check failed',
          data: {
            status: 'unhealthy',
            timestamp: new Date().toISOString(),
            version: '1.0.0'
          }
        } as ApiResponse);
      }
    });

    // Root endpoint with platform information
    this.app.get('/', (req: Request, res: Response) => {
      return res.status(200).json({
        success: true,
        data: {
          title: 'Carbon Offset Trading Platform',
          description: 'A blockchain-enabled platform for trading carbon offsets between project developers and buyers',
          version: '1.0.0',
          documentation: '/api/docs',
          healthCheck: '/health',
          endpoints: {
            auth: '/api/auth',
            wallet: '/api/wallet',
            projects: '/api/projects',
            trading: '/api/trading'
          }
        }
      } as ApiResponse);
    });

    // API routes
    this.app.use('/api/auth', authRoutes);
    this.app.use('/api/wallet', walletRoutes);
    this.app.use('/api/projects', projectRoutes);
    this.app.use('/api/trading', tradingRoutes);
    
    // API documentation placeholder
    this.app.get('/api/docs', (req: Request, res: Response) => {
      return res.status(200).json({
        success: true,
        data: {
          message: 'API Documentation',
          version: '1.0.0',
          endpoints: {
            'POST /api/auth/register': 'Register new user (developer or buyer)',
            'POST /api/auth/login': 'User login',
            'POST /api/wallet/link': 'Link blockchain wallet to user account',
            'DELETE /api/wallet/unlink': 'Unlink wallet from user account',
            'GET /api/wallet/info': 'Get wallet information and balance',
            'POST /api/wallet/generate-test-wallet': 'Generate test wallet for development',
            'POST /api/projects': 'Create new carbon offset project (developers only)',
            'GET /api/projects': 'Browse all available projects',
            'GET /api/projects/my-projects': 'Get developer\'s projects',
            'GET /api/projects/:id': 'Get project details',
            'PUT /api/projects/:id': 'Update project (own projects only)',
            'DELETE /api/projects/:id': 'Delete project (own projects only)',
            'POST /api/trading/purchase': 'Purchase carbon offsets (buyers only)',
            'GET /api/trading/my-transactions': 'Get user\'s transactions',
            'GET /api/trading/transactions/:id': 'Get transaction details',
            'GET /api/trading/marketplace/stats': 'Get marketplace statistics'
          }
        }
      } as ApiResponse);
    });
  }

  private setupErrorHandling(): void {
    // 404 handler
    this.app.use(notFoundHandler);
    
    // Global error handler
    this.app.use(errorHandler);
  }

  public async start(): Promise<void> {
    try {
      // Connect to database
      await Database.connect();
      
      // Start server
      this.app.listen(this.port, '0.0.0.0', () => {
        console.log('🚀 Carbon Offset Trading Platform Started');
        console.log(`📍 Server running on http://0.0.0.0:${this.port}`);
        console.log(`🏥 Health check: http://0.0.0.0:${this.port}/health`);
        console.log(`📚 API docs: http://0.0.0.0:${this.port}/api/docs`);
        console.log(`🌍 Environment: ${process.env.NODE_ENV || 'development'}`);
      });
      
      // Handle graceful shutdown
      this.setupGracefulShutdown();
      
    } catch (error) {
      console.error('❌ Failed to start server:', error);
      process.exit(1);
    }
  }

  private setupGracefulShutdown(): void {
    const shutdown = async (signal: string) => {
      console.log(`\n🛑 Received ${signal}. Starting graceful shutdown...`);
      
      try {
        await Database.disconnect();
        console.log('✅ Database disconnected');
        console.log('✅ Server shut down gracefully');
        process.exit(0);
      } catch (error) {
        console.error('❌ Error during shutdown:', error);
        process.exit(1);
      }
    };

    process.on('SIGINT', () => shutdown('SIGINT'));
    process.on('SIGTERM', () => shutdown('SIGTERM'));
  }
}

// Start server if this file is run directly
if (require.main === module) {
  const server = new Server();
  server.start().catch(error => {
    console.error('❌ Failed to start application:', error);
    process.exit(1);
  });
}

export default Server;